@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_pkt.c,v 1.33 2020/10/14 16:57:33 jsing Exp $ */
+/* $OpenBSD: ssl_pkt.c,v 1.40 2021/03/29 16:46:09 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -227,14 +227,14 @@ ssl3_read_n(SSL *s, int n, int max, int extend)
 		return -1;
 	}
 
-	if (!s->internal->read_ahead) {
-		/* ignore max parameter */
-		max = n;
-	} else {
+	if (s->internal->read_ahead || SSL_is_dtls(s)) {
 		if (max < n)
 			max = n;
 		if (max > (int)(rb->len - rb->offset))
 			max = rb->len - rb->offset;
+	} else {
+		/* ignore max parameter */
+		max = n;
 	}
 
 	while (left < n) {
@@ -370,13 +370,14 @@ ssl3_get_record(SSL *s)
 
 		/* Lets check version */
 		if (!s->internal->first_packet && ssl_version != s->version) {
-			SSLerror(s, SSL_R_WRONG_VERSION_NUMBER);
 			if ((s->version & 0xFF00) == (ssl_version & 0xFF00) &&
-			    !s->internal->enc_write_ctx && !s->internal->write_hash)
+			    !tls12_record_layer_write_protected(s->internal->rl)) {
 				/* Send back error using their minor version number :-) */
 				s->version = ssl_version;
+			}
+			SSLerror(s, SSL_R_WRONG_VERSION_NUMBER);
 			al = SSL_AD_PROTOCOL_VERSION;
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if ((ssl_version >> 8) != SSL3_VERSION_MAJOR) {
@@ -387,7 +388,7 @@ ssl3_get_record(SSL *s)
 		if (rr->length > rb->len - SSL3_RT_HEADER_LENGTH) {
 			al = SSL_AD_RECORD_OVERFLOW;
 			SSLerror(s, SSL_R_PACKET_LENGTH_TOO_LONG);
-			goto f_err;
+			goto fatal_err;
 		}
 	}
 
@@ -418,7 +419,7 @@ ssl3_get_record(SSL *s)
 			SSLerror(s, SSL_R_DECRYPTION_FAILED_OR_BAD_RECORD_MAC);
 
 		al = alert_desc;
-		goto f_err;
+		goto fatal_err;
 	}
 
 	rr->data = out;
@@ -449,7 +450,7 @@ ssl3_get_record(SSL *s)
 
 	return (1);
 
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	return (ret);
@@ -560,8 +561,9 @@ do_ssl3_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 	 * bytes and record version number > TLS 1.0.
 	 */
 	version = s->version;
-	if (S3I(s)->hs.state == SSL3_ST_CW_CLNT_HELLO_B && !s->internal->renegotiate &&
-	    TLS1_get_version(s) > TLS1_VERSION)
+	if (S3I(s)->hs.state == SSL3_ST_CW_CLNT_HELLO_B &&
+	    !s->internal->renegotiate &&
+	    S3I(s)->hs.our_max_tls_version > TLS1_VERSION)
 		version = TLS1_VERSION;
 
 	/*
@@ -569,8 +571,7 @@ do_ssl3_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 	 * (see http://www.openssl.org/~bodo/tls-cbc.txt). Note that this
 	 * is unnecessary for AEAD.
 	 */
-	if (sess != NULL && s->internal->enc_write_ctx != NULL &&
-	    EVP_MD_CTX_md(s->internal->write_hash) != NULL) {
+	if (sess != NULL && tls12_record_layer_write_protected(s->internal->rl)) {
 		if (S3I(s)->need_empty_fragments &&
 		    !S3I(s)->empty_fragment_done &&
 		    type == SSL3_RT_APPLICATION_DATA)
@@ -760,7 +761,7 @@ ssl3_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		}
 	}
 
-start:
+ start:
 	/*
 	 * Do not process more than three consecutive records, otherwise the
 	 * peer can cause us to loop indefinitely. Instead, return with an
@@ -798,7 +799,7 @@ start:
 	    && (rr->type != SSL3_RT_HANDSHAKE)) {
 		al = SSL_AD_UNEXPECTED_MESSAGE;
 		SSLerror(s, SSL_R_DATA_BETWEEN_CCS_AND_FINISHED);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	/* If the other end has shut down, throw anything we read away
@@ -814,11 +815,11 @@ start:
 	if (type == rr->type) {
 		/* make sure that we are not getting application data when we
 		 * are doing a handshake for the first time */
-		if (SSL_in_init(s) && (type == SSL3_RT_APPLICATION_DATA) &&
-			(s->enc_read_ctx == NULL)) {
+		if (SSL_in_init(s) && type == SSL3_RT_APPLICATION_DATA &&
+		    !tls12_record_layer_read_protected(s->internal->rl)) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_APP_DATA_IN_HANDSHAKE);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if (len <= 0)
@@ -900,7 +901,7 @@ start:
 		    (S3I(s)->handshake_fragment[3] != 0)) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_BAD_HELLO_REQUEST);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if (s->internal->msg_callback)
@@ -944,7 +945,7 @@ start:
 	    S3I(s)->handshake_fragment[0] == SSL3_MT_CLIENT_HELLO &&
 	    (s->internal->options & SSL_OP_NO_CLIENT_RENEGOTIATION)) {
 		al = SSL_AD_NO_RENEGOTIATION;
-		goto f_err;
+		goto fatal_err;
 	}
 	/* If we are a server and get a client hello when renegotiation isn't
 	 * allowed send back a no renegotiation alert and carry on.
@@ -999,7 +1000,7 @@ start:
 			else if (alert_descr == SSL_AD_NO_RENEGOTIATION) {
 				al = SSL_AD_HANDSHAKE_FAILURE;
 				SSLerror(s, SSL_R_NO_RENEGOTIATION);
-				goto f_err;
+				goto fatal_err;
 			}
 		} else if (alert_level == SSL3_AL_FATAL) {
 			s->internal->rwstate = SSL_NOTHING;
@@ -1013,7 +1014,7 @@ start:
 		} else {
 			al = SSL_AD_ILLEGAL_PARAMETER;
 			SSLerror(s, SSL_R_UNKNOWN_ALERT_TYPE);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		goto start;
@@ -1033,21 +1034,21 @@ start:
 			(rr->data[0] != SSL3_MT_CCS)) {
 			al = SSL_AD_ILLEGAL_PARAMETER;
 			SSLerror(s, SSL_R_BAD_CHANGE_CIPHER_SPEC);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		/* Check we have a cipher to change to */
-		if (S3I(s)->hs.new_cipher == NULL) {
+		if (S3I(s)->hs.cipher == NULL) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_CCS_RECEIVED_EARLY);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		/* Check that we should be receiving a Change Cipher Spec. */
 		if (!(s->s3->flags & SSL3_FLAGS_CCS_OK)) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_CCS_RECEIVED_EARLY);
-			goto f_err;
+			goto fatal_err;
 		}
 		s->s3->flags &= ~SSL3_FLAGS_CCS_OK;
 
@@ -1108,7 +1109,7 @@ start:
 		}
 		al = SSL_AD_UNEXPECTED_MESSAGE;
 		SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-		goto f_err;
+		goto fatal_err;
 	case SSL3_RT_CHANGE_CIPHER_SPEC:
 	case SSL3_RT_ALERT:
 	case SSL3_RT_HANDSHAKE:
@@ -1117,7 +1118,7 @@ start:
 		 * should not happen when type != rr->type */
 		al = SSL_AD_UNEXPECTED_MESSAGE;
 		SSLerror(s, ERR_R_INTERNAL_ERROR);
-		goto f_err;
+		goto fatal_err;
 	case SSL3_RT_APPLICATION_DATA:
 		/* At this point, we were expecting handshake data,
 		 * but have application data.  If the library was
@@ -1139,14 +1140,14 @@ start:
 		} else {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-			goto f_err;
+			goto fatal_err;
 		}
 	}
 	/* not reached */
 
-f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
-err:
+ err:
 	return (-1);
 }
 
@@ -1162,14 +1163,14 @@ ssl3_do_change_cipher_spec(SSL *s)
 	else
 		i = SSL3_CHANGE_CIPHER_CLIENT_READ;
 
-	if (S3I(s)->hs.key_block == NULL) {
+	if (S3I(s)->hs.tls12.key_block == NULL) {
 		if (s->session == NULL || s->session->master_key_length == 0) {
 			/* might happen if dtls1_read_bytes() calls this */
 			SSLerror(s, SSL_R_CCS_RECEIVED_EARLY);
 			return (0);
 		}
 
-		s->session->cipher = S3I(s)->hs.new_cipher;
+		s->session->cipher = S3I(s)->hs.cipher;
 		if (!tls1_setup_key_block(s))
 			return (0);
 	}
@@ -1189,12 +1190,12 @@ ssl3_do_change_cipher_spec(SSL *s)
 	}
 
 	i = tls1_final_finish_mac(s, sender, slen,
-	    S3I(s)->tmp.peer_finish_md);
+	    S3I(s)->hs.peer_finished);
 	if (i == 0) {
 		SSLerror(s, ERR_R_INTERNAL_ERROR);
 		return 0;
 	}
-	S3I(s)->tmp.peer_finish_md_len = i;
+	S3I(s)->hs.peer_finished_len = i;
 
 	return (1);
 }
