@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_srvr.c,v 1.88 2020/10/14 16:57:33 jsing Exp $ */
+/* $OpenBSD: ssl_srvr.c,v 1.101 2021/03/29 16:56:20 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -213,20 +213,19 @@ ssl3_accept(SSL *s)
 			if (cb != NULL)
 				cb(s, SSL_CB_HANDSHAKE_START, 1);
 
-			if (SSL_is_dtls(s)) {
-				if ((s->version & 0xff00) != (DTLS1_VERSION & 0xff00)) {
-					SSLerror(s, ERR_R_INTERNAL_ERROR);
-					ret = -1;
-					goto end;
-				}
-			} else {
-				if ((s->version >> 8) != 3) {
-					SSLerror(s, ERR_R_INTERNAL_ERROR);
-					ret = -1;
-					goto end;
-				}
+			if (!ssl_legacy_stack_version(s, s->version)) {
+				SSLerror(s, ERR_R_INTERNAL_ERROR);
+				ret = -1;
+				goto end;
+			}
+
+			if (!ssl_supported_tls_version_range(s,
+			    &S3I(s)->hs.our_min_tls_version,
+			    &S3I(s)->hs.our_max_tls_version)) {
+				SSLerror(s, SSL_R_NO_PROTOCOLS_AVAILABLE);
+				ret = -1;
+				goto end;
 			}
-			s->internal->type = SSL_ST_ACCEPT;
 
 			if (!ssl3_setup_init_buffer(s)) {
 				ret = -1;
@@ -289,15 +288,17 @@ ssl3_accept(SSL *s)
 			if (ret <= 0)
 				goto end;
 			if (SSL_is_dtls(s))
-				S3I(s)->hs.next_state = SSL3_ST_SR_CLNT_HELLO_A;
+				S3I(s)->hs.tls12.next_state = SSL3_ST_SR_CLNT_HELLO_A;
 			else
-				S3I(s)->hs.next_state = SSL3_ST_SW_HELLO_REQ_C;
+				S3I(s)->hs.tls12.next_state = SSL3_ST_SW_HELLO_REQ_C;
 			S3I(s)->hs.state = SSL3_ST_SW_FLUSH;
 			s->internal->init_num = 0;
 
-			if (!tls1_transcript_init(s)) {
-				ret = -1;
-				goto end;
+			if (SSL_is_dtls(s)) {
+				if (!tls1_transcript_init(s)) {
+					ret = -1;
+					goto end;
+				}
 			}
 			break;
 
@@ -328,9 +329,8 @@ ssl3_accept(SSL *s)
 				 * stateless while listening.
 				 */
 				if (listen) {
-					memcpy(S3I(s)->write_sequence,
-					    S3I(s)->read_sequence,
-					    sizeof(S3I(s)->write_sequence));
+					tls12_record_layer_reflect_seq_num(
+					    s->internal->rl);
 				}
 
 				/* If we're just listening, stop here */
@@ -365,7 +365,7 @@ ssl3_accept(SSL *s)
 			if (ret <= 0)
 				goto end;
 			S3I(s)->hs.state = SSL3_ST_SW_FLUSH;
-			S3I(s)->hs.next_state = SSL3_ST_SR_CLNT_HELLO_A;
+			S3I(s)->hs.tls12.next_state = SSL3_ST_SR_CLNT_HELLO_A;
 
 			/* HelloVerifyRequest resets Finished MAC. */
 			tls1_transcript_reset(s);
@@ -394,7 +394,7 @@ ssl3_accept(SSL *s)
 		case SSL3_ST_SW_CERT_A:
 		case SSL3_ST_SW_CERT_B:
 			/* Check if it is anon DH or anon ECDH. */
-			if (!(S3I(s)->hs.new_cipher->algorithm_auth &
+			if (!(S3I(s)->hs.cipher->algorithm_auth &
 			    SSL_aNULL)) {
 				if (SSL_is_dtls(s))
 					dtls1_start_timer(s);
@@ -414,7 +414,7 @@ ssl3_accept(SSL *s)
 
 		case SSL3_ST_SW_KEY_EXCH_A:
 		case SSL3_ST_SW_KEY_EXCH_B:
-			alg_k = S3I(s)->hs.new_cipher->algorithm_mkey;
+			alg_k = S3I(s)->hs.cipher->algorithm_mkey;
 
 			/*
 			 * Only send if using a DH key exchange.
@@ -459,7 +459,7 @@ ssl3_accept(SSL *s)
 			if (!(s->verify_mode & SSL_VERIFY_PEER) ||
 			    ((s->session->peer != NULL) &&
 			     (s->verify_mode & SSL_VERIFY_CLIENT_ONCE)) ||
-			    ((S3I(s)->hs.new_cipher->algorithm_auth &
+			    ((S3I(s)->hs.cipher->algorithm_auth &
 			     SSL_aNULL) && !(s->verify_mode &
 			     SSL_VERIFY_FAIL_IF_NO_PEER_CERT))) {
 				/* No cert request. */
@@ -488,7 +488,7 @@ ssl3_accept(SSL *s)
 			ret = ssl3_send_server_done(s);
 			if (ret <= 0)
 				goto end;
-			S3I(s)->hs.next_state = SSL3_ST_SR_CERT_A;
+			S3I(s)->hs.tls12.next_state = SSL3_ST_SR_CERT_A;
 			S3I(s)->hs.state = SSL3_ST_SW_FLUSH;
 			s->internal->init_num = 0;
 			break;
@@ -510,14 +510,14 @@ ssl3_accept(SSL *s)
 					/* If the write error was fatal, stop trying. */
 					if (!BIO_should_retry(s->wbio)) {
 						s->internal->rwstate = SSL_NOTHING;
-						S3I(s)->hs.state = S3I(s)->hs.next_state;
+						S3I(s)->hs.state = S3I(s)->hs.tls12.next_state;
 					}
 				}
 				ret = -1;
 				goto end;
 			}
 			s->internal->rwstate = SSL_NOTHING;
-			S3I(s)->hs.state = S3I(s)->hs.next_state;
+			S3I(s)->hs.state = S3I(s)->hs.tls12.next_state;
 			break;
 
 		case SSL3_ST_SR_CERT_A:
@@ -542,7 +542,7 @@ ssl3_accept(SSL *s)
 				s->internal->init_num = 0;
 			}
 
-			alg_k = S3I(s)->hs.new_cipher->algorithm_mkey;
+			alg_k = S3I(s)->hs.cipher->algorithm_mkey;
 			if (ret == 2) {
 				/*
 				 * For the ECDH ciphersuites when
@@ -641,7 +641,7 @@ ssl3_accept(SSL *s)
 
 		case SSL3_ST_SW_CHANGE_A:
 		case SSL3_ST_SW_CHANGE_B:
-			s->session->cipher = S3I(s)->hs.new_cipher;
+			s->session->cipher = S3I(s)->hs.cipher;
 			if (!tls1_setup_key_block(s)) {
 				ret = -1;
 				goto end;
@@ -674,10 +674,10 @@ ssl3_accept(SSL *s)
 				goto end;
 			S3I(s)->hs.state = SSL3_ST_SW_FLUSH;
 			if (s->internal->hit) {
-				S3I(s)->hs.next_state = SSL3_ST_SR_FINISHED_A;
+				S3I(s)->hs.tls12.next_state = SSL3_ST_SR_FINISHED_A;
 				tls1_transcript_free(s);
 			} else
-				S3I(s)->hs.next_state = SSL_ST_OK;
+				S3I(s)->hs.tls12.next_state = SSL_ST_OK;
 			s->internal->init_num = 0;
 			break;
 
@@ -749,7 +749,7 @@ ssl3_accept(SSL *s)
 		}
 		skip = 0;
 	}
-end:
+ end:
 	/* BIO_flush(s->wbio); */
 	s->internal->in_handshake--;
 	if (cb != NULL)
@@ -799,7 +799,7 @@ ssl3_get_client_hello(SSL *s)
 	STACK_OF(SSL_CIPHER) *ciphers = NULL;
 	unsigned long alg_k;
 	const SSL_METHOD *method;
-	uint16_t max_version, shared_version;
+	uint16_t shared_version;
 
 	/*
 	 * We do this so that we will respond with our native type.
@@ -827,47 +827,51 @@ ssl3_get_client_hello(SSL *s)
 
 	/* Parse client hello up until the extensions (if any). */
 	if (!CBS_get_u16(&cbs, &client_version))
-		goto truncated;
+		goto decode_err;
 	if (!CBS_get_bytes(&cbs, &client_random, SSL3_RANDOM_SIZE))
-		goto truncated;
+		goto decode_err;
 	if (!CBS_get_u8_length_prefixed(&cbs, &session_id))
-		goto truncated;
+		goto decode_err;
 	if (CBS_len(&session_id) > SSL3_SESSION_ID_SIZE) {
 		al = SSL_AD_ILLEGAL_PARAMETER;
 		SSLerror(s, SSL_R_SSL3_SESSION_ID_TOO_LONG);
-		goto f_err;
+		goto fatal_err;
 	}
 	if (SSL_is_dtls(s)) {
 		if (!CBS_get_u8_length_prefixed(&cbs, &cookie))
-			goto truncated;
+			goto decode_err;
 	}
 	if (!CBS_get_u16_length_prefixed(&cbs, &cipher_suites))
-		goto truncated;
+		goto decode_err;
 	if (!CBS_get_u8_length_prefixed(&cbs, &compression_methods))
-		goto truncated;
+		goto decode_err;
 
 	/*
 	 * Use version from inside client hello, not from record header.
 	 * (may differ: see RFC 2246, Appendix E, second paragraph)
 	 */
-	if (!ssl_downgrade_max_version(s, &max_version))
-		goto err;
-	if (ssl_max_shared_version(s, client_version, &shared_version) != 1) {
-		SSLerror(s, SSL_R_WRONG_VERSION_NUMBER);
+	if (!ssl_max_shared_version(s, client_version, &shared_version)) {
 		if ((s->client_version >> 8) == SSL3_VERSION_MAJOR &&
-		    !s->internal->enc_write_ctx && !s->internal->write_hash) {
+		    !tls12_record_layer_write_protected(s->internal->rl)) {
 			/*
 			 * Similar to ssl3_get_record, send alert using remote
 			 * version number.
 			 */
 			s->version = s->client_version;
 		}
+		SSLerror(s, SSL_R_WRONG_VERSION_NUMBER);
 		al = SSL_AD_PROTOCOL_VERSION;
-		goto f_err;
+		goto fatal_err;
 	}
 	s->client_version = client_version;
 	s->version = shared_version;
 
+	S3I(s)->hs.negotiated_tls_version = ssl_tls_version(shared_version);
+	if (S3I(s)->hs.negotiated_tls_version == 0) {
+		SSLerror(s, ERR_R_INTERNAL_ERROR);
+		goto err;
+	}
+
 	if ((method = ssl_get_method(shared_version)) == NULL) {
 		SSLerror(s, ERR_R_INTERNAL_ERROR);
 		goto err;
@@ -920,7 +924,7 @@ ssl3_get_client_hello(SSL *s)
 		if (i == 1) { /* previous session */
 			s->internal->hit = 1;
 		} else if (i == -1)
-			goto f_err;
+			goto fatal_err;
 		else {
 			/* i == 0 */
 			if (!ssl_get_new_session(s, 1))
@@ -937,7 +941,7 @@ ssl3_get_client_hello(SSL *s)
 		if (CBS_len(&cookie) > sizeof(D1I(s)->rcvd_cookie)) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_COOKIE_MISMATCH);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		/* Verify the cookie if appropriate option is set. */
@@ -955,7 +959,7 @@ ssl3_get_client_hello(SSL *s)
 				    D1I(s)->rcvd_cookie, cookie_len) == 0) {
 					al = SSL_AD_HANDSHAKE_FAILURE;
 					SSLerror(s, SSL_R_COOKIE_MISMATCH);
-					goto f_err;
+					goto fatal_err;
 				}
 				/* else cookie verification succeeded */
 			/* XXX - can d1->cookie_len > sizeof(rcvd_cookie) ? */
@@ -964,7 +968,7 @@ ssl3_get_client_hello(SSL *s)
 				/* default verification */
 				al = SSL_AD_HANDSHAKE_FAILURE;
 				SSLerror(s, SSL_R_COOKIE_MISMATCH);
-				goto f_err;
+				goto fatal_err;
 			}
 			cookie_valid = 1;
 		}
@@ -975,7 +979,7 @@ ssl3_get_client_hello(SSL *s)
 		/* we need a cipher if we are not resuming a session */
 		al = SSL_AD_ILLEGAL_PARAMETER;
 		SSLerror(s, SSL_R_NO_CIPHERS_SPECIFIED);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (CBS_len(&cipher_suites) > 0) {
@@ -1004,32 +1008,32 @@ ssl3_get_client_hello(SSL *s)
 			 */
 			al = SSL_AD_ILLEGAL_PARAMETER;
 			SSLerror(s, SSL_R_REQUIRED_CIPHER_MISSING);
-			goto f_err;
+			goto fatal_err;
 		}
 	}
 
 	comp_null = 0;
 	while (CBS_len(&compression_methods) > 0) {
 		if (!CBS_get_u8(&compression_methods, &comp_method))
-			goto truncated;
+			goto decode_err;
 		if (comp_method == 0)
 			comp_null = 1;
 	}
 	if (comp_null == 0) {
 		al = SSL_AD_DECODE_ERROR;
 		SSLerror(s, SSL_R_NO_COMPRESSION_SPECIFIED);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (!tlsext_server_parse(s, SSL_TLSEXT_MSG_CH, &cbs, &al)) {
 		SSLerror(s, SSL_R_PARSE_TLSEXT);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (!S3I(s)->renegotiate_seen && s->internal->renegotiate) {
 		al = SSL_AD_HANDSHAKE_FAILURE;
 		SSLerror(s, SSL_R_UNSAFE_LEGACY_RENEGOTIATION_DISABLED);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (ssl_check_clienthello_tlsext_early(s) <= 0) {
@@ -1045,8 +1049,8 @@ ssl3_get_client_hello(SSL *s)
 	 */
 	arc4random_buf(s->s3->server_random, SSL3_RANDOM_SIZE);
 
-	if (!SSL_is_dtls(s) && max_version >= TLS1_2_VERSION &&
-	    s->version < max_version) {
+	if (S3I(s)->hs.our_max_tls_version >= TLS1_2_VERSION &&
+	    S3I(s)->hs.negotiated_tls_version < S3I(s)->hs.our_max_tls_version) {
 		/*
 		 * RFC 8446 section 4.1.3. If we are downgrading from TLS 1.3
 		 * we must set the last 8 bytes of the server random to magical
@@ -1055,7 +1059,7 @@ ssl3_get_client_hello(SSL *s)
 		 */
 		size_t index = SSL3_RANDOM_SIZE - sizeof(tls13_downgrade_12);
 		uint8_t *magic = &s->s3->server_random[index];
-		if (s->version == TLS1_2_VERSION) {
+		if (S3I(s)->hs.negotiated_tls_version == TLS1_2_VERSION) {
 			/* Indicate we chose to downgrade to 1.2. */
 			memcpy(magic, tls13_downgrade_12,
 			    sizeof(tls13_downgrade_12));
@@ -1086,7 +1090,7 @@ ssl3_get_client_hello(SSL *s)
 			if (pref_cipher == NULL) {
 				al = SSL_AD_HANDSHAKE_FAILURE;
 				SSLerror(s, SSL_R_NO_SHARED_CIPHER);
-				goto f_err;
+				goto fatal_err;
 			}
 
 			s->session->cipher = pref_cipher;
@@ -1107,7 +1111,7 @@ ssl3_get_client_hello(SSL *s)
 		if (ciphers == NULL) {
 			al = SSL_AD_ILLEGAL_PARAMETER;
 			SSLerror(s, SSL_R_NO_CIPHERS_PASSED);
-			goto f_err;
+			goto fatal_err;
 		}
 		ciphers = NULL;
 		c = ssl3_choose_cipher(s, s->session->ciphers,
@@ -1116,17 +1120,17 @@ ssl3_get_client_hello(SSL *s)
 		if (c == NULL) {
 			al = SSL_AD_HANDSHAKE_FAILURE;
 			SSLerror(s, SSL_R_NO_SHARED_CIPHER);
-			goto f_err;
+			goto fatal_err;
 		}
-		S3I(s)->hs.new_cipher = c;
+		S3I(s)->hs.cipher = c;
 	} else {
-		S3I(s)->hs.new_cipher = s->session->cipher;
+		S3I(s)->hs.cipher = s->session->cipher;
 	}
 
 	if (!tls1_transcript_hash_init(s))
 		goto err;
 
-	alg_k = S3I(s)->hs.new_cipher->algorithm_mkey;
+	alg_k = S3I(s)->hs.cipher->algorithm_mkey;
 	if (!(SSL_USE_SIGALGS(s) || (alg_k & SSL_kGOST)) ||
 	    !(s->verify_mode & SSL_VERIFY_PEER))
 		tls1_transcript_free(s);
@@ -1134,13 +1138,13 @@ ssl3_get_client_hello(SSL *s)
 	/*
 	 * We now have the following setup.
 	 * client_random
-	 * cipher_list 		- our prefered list of ciphers
-	 * ciphers 		- the clients prefered list of ciphers
+	 * cipher_list		- our prefered list of ciphers
+	 * ciphers		- the clients prefered list of ciphers
 	 * compression		- basically ignored right now
 	 * ssl version is set	- sslv3
 	 * s->session		- The ssl session has been setup.
 	 * s->internal->hit		- session reuse flag
-	 * s->hs.new_cipher	- the new cipher to use.
+	 * s->hs.cipher	- the new cipher to use.
 	 */
 
 	/* Handles TLS extensions that we couldn't check earlier */
@@ -1152,13 +1156,13 @@ ssl3_get_client_hello(SSL *s)
 	ret = cookie_valid ? 2 : 1;
 
 	if (0) {
-truncated:
+ decode_err:
 		al = SSL_AD_DECODE_ERROR;
 		SSLerror(s, SSL_R_BAD_PACKET_LENGTH);
-f_err:
+ fatal_err:
 		ssl3_send_alert(s, SSL3_AL_FATAL, al);
 	}
-err:
+ err:
 	sk_SSL_CIPHER_free(ciphers);
 
 	return (ret);
@@ -1179,10 +1183,15 @@ ssl3_send_dtls_hello_verify_request(SSL *s)
 			return 0;
 		}
 
+		/*
+		 * Per RFC 6347 section 4.2.1, the HelloVerifyRequest should
+		 * always contain DTLSv1.0 regardless of the version that is
+		 * going to be negotiated.
+		 */
 		if (!ssl3_handshake_msg_start(s, &cbb, &verify,
 		    DTLS1_MT_HELLO_VERIFY_REQUEST))
 			goto err;
-		if (!CBB_add_u16(&verify, s->version))
+		if (!CBB_add_u16(&verify, DTLS1_VERSION))
 			goto err;
 		if (!CBB_add_u8_length_prefixed(&verify, &cookie))
 			goto err;
@@ -1256,7 +1265,7 @@ ssl3_send_server_hello(SSL *s)
 
 		/* Cipher suite. */
 		if (!CBB_add_u16(&server_hello,
-		    ssl3_cipher_get_value(S3I(s)->hs.new_cipher)))
+		    ssl3_cipher_get_value(S3I(s)->hs.cipher)))
 			goto err;
 
 		/* Compression method (null). */
@@ -1320,19 +1329,19 @@ ssl3_send_server_kex_dhe(SSL *s, CBB *cbb)
 		if ((dhp = ssl_get_auto_dh(s)) == NULL) {
 			al = SSL_AD_INTERNAL_ERROR;
 			SSLerror(s, ERR_R_INTERNAL_ERROR);
-			goto f_err;
+			goto fatal_err;
 		}
 	} else
 		dhp = s->cert->dh_tmp;
 
 	if (dhp == NULL && s->cert->dh_tmp_cb != NULL)
 		dhp = s->cert->dh_tmp_cb(s, 0,
-		    SSL_C_PKEYLENGTH(S3I(s)->hs.new_cipher));
+		    SSL_C_PKEYLENGTH(S3I(s)->hs.cipher));
 
 	if (dhp == NULL) {
 		al = SSL_AD_HANDSHAKE_FAILURE;
 		SSLerror(s, SSL_R_MISSING_TMP_DH_KEY);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (S3I(s)->tmp.dh != NULL) {
@@ -1378,7 +1387,7 @@ ssl3_send_server_kex_dhe(SSL *s, CBB *cbb)
 
 	return (1);
 
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	return (-1);
@@ -1409,7 +1418,7 @@ ssl3_send_server_kex_ecdhe_ecp(SSL *s, int nid, CBB *cbb)
 	if ((S3I(s)->tmp.ecdh = EC_KEY_new()) == NULL) {
 		al = SSL_AD_HANDSHAKE_FAILURE;
 		SSLerror(s, SSL_R_MISSING_TMP_ECDH_KEY);
-		goto f_err;
+		goto fatal_err;
 	}
 	S3I(s)->tmp.ecdh_nid = nid;
 	ecdh = S3I(s)->tmp.ecdh;
@@ -1439,7 +1448,7 @@ ssl3_send_server_kex_ecdhe_ecp(SSL *s, int nid, CBB *cbb)
 
 	return (1);
 
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	return (-1);
@@ -1535,7 +1544,7 @@ ssl3_send_server_key_exchange(SSL *s)
 		if (!CBB_init(&cbb_params, 0))
 			goto err;
 
-		type = S3I(s)->hs.new_cipher->algorithm_mkey;
+		type = S3I(s)->hs.cipher->algorithm_mkey;
 		if (type & SSL_kDHE) {
 			if (ssl3_send_server_kex_dhe(s, &cbb_params) != 1)
 				goto err;
@@ -1545,7 +1554,7 @@ ssl3_send_server_key_exchange(SSL *s)
 		} else {
 			al = SSL_AD_HANDSHAKE_FAILURE;
 			SSLerror(s, SSL_R_UNKNOWN_KEY_EXCHANGE_TYPE);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if (!CBB_finish(&cbb_params, &params, &params_len))
@@ -1555,11 +1564,11 @@ ssl3_send_server_key_exchange(SSL *s)
 			goto err;
 
 		/* Add signature unless anonymous. */
-		if (!(S3I(s)->hs.new_cipher->algorithm_auth & SSL_aNULL)) {
-			if ((pkey = ssl_get_sign_pkey(s, S3I(s)->hs.new_cipher,
+		if (!(S3I(s)->hs.cipher->algorithm_auth & SSL_aNULL)) {
+			if ((pkey = ssl_get_sign_pkey(s, S3I(s)->hs.cipher,
 			    &md, &sigalg)) == NULL) {
 				al = SSL_AD_DECODE_ERROR;
-				goto f_err;
+				goto fatal_err;
 			}
 
 			/* Send signature algorithm. */
@@ -1567,7 +1576,7 @@ ssl3_send_server_key_exchange(SSL *s)
 				if (!CBB_add_u16(&server_kex, sigalg->value)) {
 					al = SSL_AD_INTERNAL_ERROR;
 					SSLerror(s, ERR_R_INTERNAL_ERROR);
-					goto f_err;
+					goto fatal_err;
 				}
 			}
 
@@ -1630,7 +1639,7 @@ ssl3_send_server_key_exchange(SSL *s)
 
 	return (ssl3_handshake_write(s));
 
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	CBB_cleanup(&cbb_params);
@@ -1721,6 +1730,8 @@ ssl3_get_client_kex_rsa(SSL *s, CBS *cbs)
 	int al = -1;
 
 	arc4random_buf(fakekey, sizeof(fakekey));
+
+	/* XXX - peer max protocol version. */
 	fakekey[0] = s->client_version >> 8;
 	fakekey[1] = s->client_version & 0xff;
 
@@ -1729,7 +1740,7 @@ ssl3_get_client_kex_rsa(SSL *s, CBS *cbs)
 	    (pkey->pkey.rsa == NULL)) {
 		al = SSL_AD_HANDSHAKE_FAILURE;
 		SSLerror(s, SSL_R_MISSING_RSA_CERTIFICATE);
-		goto f_err;
+		goto fatal_err;
 	}
 	rsa = pkey->pkey.rsa;
 
@@ -1741,7 +1752,7 @@ ssl3_get_client_kex_rsa(SSL *s, CBS *cbs)
 	p = pms;
 
 	if (!CBS_get_u16_length_prefixed(cbs, &enc_pms))
-		goto truncated;
+		goto decode_err;
 	if (CBS_len(cbs) != 0 || CBS_len(&enc_pms) != RSA_size(rsa)) {
 		SSLerror(s, SSL_R_TLS_RSA_ENCRYPTED_VALUE_LENGTH_IS_WRONG);
 		goto err;
@@ -1757,6 +1768,7 @@ ssl3_get_client_kex_rsa(SSL *s, CBS *cbs)
 		/* SSLerror(s, SSL_R_BAD_RSA_DECRYPT); */
 	}
 
+	/* XXX - peer max version. */
 	if ((al == -1) && !((pms[0] == (s->client_version >> 8)) &&
 	    (pms[1] == (s->client_version & 0xff)))) {
 		/*
@@ -1789,16 +1801,16 @@ ssl3_get_client_kex_rsa(SSL *s, CBS *cbs)
 
 	s->session->master_key_length =
 	    tls1_generate_master_secret(s,
-	        s->session->master_key, p, SSL_MAX_MASTER_KEY_LENGTH);
+		s->session->master_key, p, SSL_MAX_MASTER_KEY_LENGTH);
 
 	freezero(pms, pms_len);
 
 	return (1);
 
- truncated:
+ decode_err:
 	al = SSL_AD_DECODE_ERROR;
 	SSLerror(s, SSL_R_BAD_PACKET_LENGTH);
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	freezero(pms, pms_len);
@@ -1817,14 +1829,14 @@ ssl3_get_client_kex_dhe(SSL *s, CBS *cbs)
 	DH *dh;
 
 	if (!CBS_get_u16_length_prefixed(cbs, &dh_Yc))
-		goto truncated;
+		goto decode_err;
 	if (CBS_len(cbs) != 0)
-		goto truncated;
+		goto decode_err;
 
 	if (S3I(s)->tmp.dh == NULL) {
 		al = SSL_AD_HANDSHAKE_FAILURE;
 		SSLerror(s, SSL_R_MISSING_TMP_DH_KEY);
-		goto f_err;
+		goto fatal_err;
 	}
 	dh = S3I(s)->tmp.dh;
 
@@ -1844,17 +1856,17 @@ ssl3_get_client_kex_dhe(SSL *s, CBS *cbs)
 	if (!DH_check_pub_key(dh, bn, &key_is_invalid)) {
 		al = SSL_AD_INTERNAL_ERROR;
 		SSLerror(s, ERR_R_DH_LIB);
-		goto f_err;
+		goto fatal_err;
 	}
 	if (key_is_invalid) {
 		al = SSL_AD_ILLEGAL_PARAMETER;
 		SSLerror(s, ERR_R_DH_LIB);
-		goto f_err;
+		goto fatal_err;
 	}
 	if ((key_len = DH_compute_key(key, bn, dh)) <= 0) {
 		al = SSL_AD_INTERNAL_ERROR;
 		SSLerror(s, ERR_R_DH_LIB);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	s->session->master_key_length = tls1_generate_master_secret(s,
@@ -1868,10 +1880,10 @@ ssl3_get_client_kex_dhe(SSL *s, CBS *cbs)
 
 	return (1);
 
- truncated:
+ decode_err:
 	al = SSL_AD_DECODE_ERROR;
 	SSLerror(s, SSL_R_BAD_PACKET_LENGTH);
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	freezero(key, key_size);
@@ -1971,7 +1983,7 @@ ssl3_get_client_kex_ecdhe_ecx(SSL *s, CBS *cbs)
 static int
 ssl3_get_client_kex_ecdhe(SSL *s, CBS *cbs)
 {
-        if (S3I(s)->tmp.x25519 != NULL)
+	if (S3I(s)->tmp.x25519 != NULL)
 		return ssl3_get_client_kex_ecdhe_ecx(s, cbs);
 
 	return ssl3_get_client_kex_ecdhe_ecp(s, cbs);
@@ -1990,7 +2002,7 @@ ssl3_get_client_kex_gost(SSL *s, CBS *cbs)
 	int ret = 0;
 
 	/* Get our certificate private key*/
-	alg_a = S3I(s)->hs.new_cipher->algorithm_auth;
+	alg_a = S3I(s)->hs.cipher->algorithm_auth;
 	if (alg_a & SSL_aGOST01)
 		pk = s->cert->pkeys[SSL_PKEY_GOST01].privatekey;
 
@@ -2014,9 +2026,9 @@ ssl3_get_client_kex_gost(SSL *s, CBS *cbs)
 
 	/* Decrypt session key */
 	if (!CBS_get_asn1(cbs, &gostblob, CBS_ASN1_SEQUENCE))
-		goto truncated;
+		goto decode_err;
 	if (CBS_len(cbs) != 0)
-		goto truncated;
+		goto decode_err;
 	if (EVP_PKEY_decrypt(pkey_ctx, premaster_secret, &outlen,
 	    CBS_data(&gostblob), CBS_len(&gostblob)) <= 0) {
 		SSLerror(s, SSL_R_DECRYPTION_FAILED);
@@ -2042,7 +2054,7 @@ ssl3_get_client_kex_gost(SSL *s, CBS *cbs)
 	else
 		goto err;
 
- truncated:
+ decode_err:
 	al = SSL_AD_DECODE_ERROR;
 	SSLerror(s, SSL_R_BAD_PACKET_LENGTH);
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
@@ -2069,7 +2081,7 @@ ssl3_get_client_key_exchange(SSL *s)
 
 	CBS_init(&cbs, s->internal->init_msg, n);
 
-	alg_k = S3I(s)->hs.new_cipher->algorithm_mkey;
+	alg_k = S3I(s)->hs.cipher->algorithm_mkey;
 
 	if (alg_k & SSL_kRSA) {
 		if (ssl3_get_client_kex_rsa(s, &cbs) != 1)
@@ -2086,18 +2098,18 @@ ssl3_get_client_key_exchange(SSL *s)
 	} else {
 		al = SSL_AD_HANDSHAKE_FAILURE;
 		SSLerror(s, SSL_R_UNKNOWN_CIPHER_TYPE);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (CBS_len(&cbs) != 0) {
 		al = SSL_AD_DECODE_ERROR;
 		SSLerror(s, SSL_R_BAD_PACKET_LENGTH);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	return (1);
 
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	return (-1);
@@ -2142,7 +2154,7 @@ ssl3_get_cert_verify(SSL *s)
 		if (peer != NULL) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_MISSING_VERIFY_MESSAGE);
-			goto f_err;
+			goto fatal_err;
 		}
 		ret = 1;
 		goto end;
@@ -2151,19 +2163,19 @@ ssl3_get_cert_verify(SSL *s)
 	if (peer == NULL) {
 		SSLerror(s, SSL_R_NO_CLIENT_CERT_RECEIVED);
 		al = SSL_AD_UNEXPECTED_MESSAGE;
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (!(type & EVP_PKT_SIGN)) {
 		SSLerror(s, SSL_R_SIGNATURE_FOR_NON_SIGNING_CERTIFICATE);
 		al = SSL_AD_ILLEGAL_PARAMETER;
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (S3I(s)->change_cipher_spec) {
 		SSLerror(s, SSL_R_CCS_RECEIVED_EARLY);
 		al = SSL_AD_UNEXPECTED_MESSAGE;
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (!SSL_USE_SIGALGS(s)) {
@@ -2172,12 +2184,12 @@ ssl3_get_cert_verify(SSL *s)
 		if (CBS_len(&signature) > EVP_PKEY_size(pkey)) {
 			SSLerror(s, SSL_R_WRONG_SIGNATURE_SIZE);
 			al = SSL_AD_DECODE_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (CBS_len(&cbs) != 0) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_EXTRA_DATA_IN_MESSAGE);
-			goto f_err;
+			goto fatal_err;
 		}
 	}
 
@@ -2186,18 +2198,18 @@ ssl3_get_cert_verify(SSL *s)
 		uint16_t sigalg_value;
 
 		if (!CBS_get_u16(&cbs, &sigalg_value))
-			goto truncated;
+			goto decode_err;
 		if ((sigalg = ssl_sigalg(sigalg_value, tls12_sigalgs,
 		    tls12_sigalgs_len)) == NULL ||
 		    (md = sigalg->md()) == NULL) {
 			SSLerror(s, SSL_R_UNKNOWN_DIGEST);
 			al = SSL_AD_DECODE_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (!ssl_sigalg_pkey_ok(sigalg, pkey, 0)) {
 			SSLerror(s, SSL_R_WRONG_SIGNATURE_TYPE);
 			al = SSL_AD_DECODE_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if (!CBS_get_u16_length_prefixed(&cbs, &signature))
@@ -2205,48 +2217,48 @@ ssl3_get_cert_verify(SSL *s)
 		if (CBS_len(&signature) > EVP_PKEY_size(pkey)) {
 			SSLerror(s, SSL_R_WRONG_SIGNATURE_SIZE);
 			al = SSL_AD_DECODE_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (CBS_len(&cbs) != 0) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_EXTRA_DATA_IN_MESSAGE);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if (!tls1_transcript_data(s, &hdata, &hdatalen)) {
 			SSLerror(s, ERR_R_INTERNAL_ERROR);
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (!EVP_DigestVerifyInit(&mctx, &pctx, md, NULL, pkey)) {
 			SSLerror(s, ERR_R_EVP_LIB);
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if ((sigalg->flags & SIGALG_FLAG_RSA_PSS) &&
 		    (!EVP_PKEY_CTX_set_rsa_padding
 		    (pctx, RSA_PKCS1_PSS_PADDING) ||
 		    !EVP_PKEY_CTX_set_rsa_pss_saltlen(pctx, -1))) {
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (sigalg->key_type == EVP_PKEY_GOSTR01 &&
 		    EVP_PKEY_CTX_ctrl(pctx, -1, EVP_PKEY_OP_VERIFY,
 		    EVP_PKEY_CTRL_GOST_SIG_FORMAT, GOST_SIG_FORMAT_RS_LE,
 		    NULL) <= 0) {
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (!EVP_DigestVerifyUpdate(&mctx, hdata, hdatalen)) {
 			SSLerror(s, ERR_R_EVP_LIB);
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (EVP_DigestVerifyFinal(&mctx, CBS_data(&signature),
 		    CBS_len(&signature)) <= 0) {
 			al = SSL_AD_DECRYPT_ERROR;
 			SSLerror(s, SSL_R_BAD_SIGNATURE);
-			goto f_err;
+			goto fatal_err;
 		}
 	} else if (pkey->type == EVP_PKEY_RSA) {
 		verify = RSA_verify(NID_md5_sha1, S3I(s)->tmp.cert_verify_md,
@@ -2255,12 +2267,12 @@ ssl3_get_cert_verify(SSL *s)
 		if (verify < 0) {
 			al = SSL_AD_DECRYPT_ERROR;
 			SSLerror(s, SSL_R_BAD_RSA_DECRYPT);
-			goto f_err;
+			goto fatal_err;
 		}
 		if (verify == 0) {
 			al = SSL_AD_DECRYPT_ERROR;
 			SSLerror(s, SSL_R_BAD_RSA_SIGNATURE);
-			goto f_err;
+			goto fatal_err;
 		}
 	} else if (pkey->type == EVP_PKEY_EC) {
 		verify = ECDSA_verify(pkey->save_type,
@@ -2270,7 +2282,7 @@ ssl3_get_cert_verify(SSL *s)
 		if (verify <= 0) {
 			al = SSL_AD_DECRYPT_ERROR;
 			SSLerror(s, SSL_R_BAD_ECDSA_SIGNATURE);
-			goto f_err;
+			goto fatal_err;
 		}
 #ifndef OPENSSL_NO_GOST
 	} else if (pkey->type == NID_id_GostR3410_94 ||
@@ -2283,18 +2295,18 @@ ssl3_get_cert_verify(SSL *s)
 		if (!tls1_transcript_data(s, &hdata, &hdatalen)) {
 			SSLerror(s, ERR_R_INTERNAL_ERROR);
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (!EVP_PKEY_get_default_digest_nid(pkey, &nid) ||
 		    !(md = EVP_get_digestbynid(nid))) {
 			SSLerror(s, ERR_R_EVP_LIB);
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if ((pctx = EVP_PKEY_CTX_new(pkey, NULL)) == NULL) {
 			SSLerror(s, ERR_R_EVP_LIB);
 			al = SSL_AD_INTERNAL_ERROR;
-			goto f_err;
+			goto fatal_err;
 		}
 		if (!EVP_DigestInit_ex(&mctx, md, NULL) ||
 		    !EVP_DigestUpdate(&mctx, hdata, hdatalen) ||
@@ -2307,14 +2319,14 @@ ssl3_get_cert_verify(SSL *s)
 			SSLerror(s, ERR_R_EVP_LIB);
 			al = SSL_AD_INTERNAL_ERROR;
 			EVP_PKEY_CTX_free(pctx);
-			goto f_err;
+			goto fatal_err;
 		}
 		if (EVP_PKEY_verify(pctx, CBS_data(&signature),
 		    CBS_len(&signature), sigbuf, siglen) <= 0) {
 			al = SSL_AD_DECRYPT_ERROR;
 			SSLerror(s, SSL_R_BAD_SIGNATURE);
 			EVP_PKEY_CTX_free(pctx);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		EVP_PKEY_CTX_free(pctx);
@@ -2322,15 +2334,15 @@ ssl3_get_cert_verify(SSL *s)
 	} else {
 		SSLerror(s, ERR_R_INTERNAL_ERROR);
 		al = SSL_AD_UNSUPPORTED_CERTIFICATE;
-		goto f_err;
+		goto fatal_err;
 	}
 
 	ret = 1;
 	if (0) {
- truncated:
+ decode_err:
 		al = SSL_AD_DECODE_ERROR;
 		SSLerror(s, SSL_R_BAD_PACKET_LENGTH);
- f_err:
+ fatal_err:
 		ssl3_send_alert(s, SSL3_AL_FATAL, al);
 	}
  end:
@@ -2359,9 +2371,9 @@ ssl3_get_client_certificate(SSL *s)
 	if (S3I(s)->tmp.message_type == SSL3_MT_CLIENT_KEY_EXCHANGE) {
 		if ((s->verify_mode & SSL_VERIFY_PEER) &&
 		    (s->verify_mode & SSL_VERIFY_FAIL_IF_NO_PEER_CERT)) {
-		    	SSLerror(s, SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE);
+			SSLerror(s, SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE);
 			al = SSL_AD_HANDSHAKE_FAILURE;
-			goto f_err;
+			goto fatal_err;
 		}
 		/*
 		 * If tls asked for a client cert,
@@ -2371,7 +2383,7 @@ ssl3_get_client_certificate(SSL *s)
 			SSLerror(s, SSL_R_TLS_PEER_DID_NOT_RESPOND_WITH_CERTIFICATE_LIST
 			    );
 			al = SSL_AD_UNEXPECTED_MESSAGE;
-			goto f_err;
+			goto fatal_err;
 		}
 		S3I(s)->tmp.reuse_message = 1;
 		return (1);
@@ -2380,11 +2392,11 @@ ssl3_get_client_certificate(SSL *s)
 	if (S3I(s)->tmp.message_type != SSL3_MT_CERTIFICATE) {
 		al = SSL_AD_UNEXPECTED_MESSAGE;
 		SSLerror(s, SSL_R_WRONG_MESSAGE_TYPE);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	if (n < 0)
-		goto truncated;
+		goto decode_err;
 
 	CBS_init(&cbs, s->internal->init_msg, n);
 
@@ -2395,7 +2407,7 @@ ssl3_get_client_certificate(SSL *s)
 
 	if (!CBS_get_u24_length_prefixed(&cbs, &client_certs) ||
 	    CBS_len(&cbs) != 0)
-		goto truncated;
+		goto decode_err;
 
 	while (CBS_len(&client_certs) > 0) {
 		CBS cert;
@@ -2403,7 +2415,7 @@ ssl3_get_client_certificate(SSL *s)
 		if (!CBS_get_u24_length_prefixed(&client_certs, &cert)) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_CERT_LENGTH_MISMATCH);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		q = CBS_data(&cert);
@@ -2415,7 +2427,7 @@ ssl3_get_client_certificate(SSL *s)
 		if (q != CBS_data(&cert) + CBS_len(&cert)) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_CERT_LENGTH_MISMATCH);
-			goto f_err;
+			goto fatal_err;
 		}
 		if (!sk_X509_push(sk, x)) {
 			SSLerror(s, ERR_R_MALLOC_FAILURE);
@@ -2433,7 +2445,7 @@ ssl3_get_client_certificate(SSL *s)
 		    (s->verify_mode & SSL_VERIFY_FAIL_IF_NO_PEER_CERT)) {
 			SSLerror(s, SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE);
 			al = SSL_AD_HANDSHAKE_FAILURE;
-			goto f_err;
+			goto fatal_err;
 		}
 		/* No client certificate so free transcript. */
 		tls1_transcript_free(s);
@@ -2442,7 +2454,7 @@ ssl3_get_client_certificate(SSL *s)
 		if (i <= 0) {
 			al = ssl_verify_alarm_type(s->verify_result);
 			SSLerror(s, SSL_R_NO_CERTIFICATE_RETURNED);
-			goto f_err;
+			goto fatal_err;
 		}
 	}
 
@@ -2473,13 +2485,13 @@ ssl3_get_client_certificate(SSL *s)
 
 	ret = 1;
 	if (0) {
-truncated:
+ decode_err:
 		al = SSL_AD_DECODE_ERROR;
 		SSLerror(s, SSL_R_BAD_PACKET_LENGTH);
-f_err:
+ fatal_err:
 		ssl3_send_alert(s, SSL3_AL_FATAL, al);
 	}
-err:
+ err:
 	X509_free(x);
 	sk_X509_pop_free(sk, X509_free);
 
